@@ -60,19 +60,19 @@ void FontDef(unsigned char* command, void* parent)
   a = UNumRead(current+12, 1); /* length for font name */
   l = UNumRead(current+13, 1); /* device length */
   if (((struct font_entry*)parent)->type==FONT_TYPE_VF) {
-    DEBUG_PRINT((DEBUG_VF," %d %d %d",k,c,s));
+    DEBUG_PRINT(DEBUG_VF,(" %d %d %d",k,c,s));
     /* Rescale. s is relative to the actual scale /(1<<20) */
     s = (uint32_t)((uint64_t) s * (((struct font_entry*) parent)->s) 
 		   / (1<<20));
-    DEBUG_PRINT((DEBUG_VF," (%d) %d",s,d));
+    DEBUG_PRINT(DEBUG_VF,(" (%d) %d",s,d));
     /* Oddly, d differs in the DVI and the VF that my system produces */
     d = (uint32_t)((uint64_t) d * ((struct font_entry*)parent)->d
 		   / ((struct font_entry*)parent)->designsize);
-    DEBUG_PRINT((DEBUG_VF," (%d)",d));
-    DEBUG_PRINT((DEBUG_VF," %d %d '%.*s'",a,l,a+l,current+14));
+    DEBUG_PRINT(DEBUG_VF,(" (%d)",d));
+    DEBUG_PRINT(DEBUG_VF,(" %d %d '%.*s'",a,l,a+l,current+14));
 #ifdef DEBUG
   } else {
-    DEBUG_PRINT((DEBUG_DVI," %d %d %d %d %d %d '%.*s'",k,c,s,d,a,l,
+    DEBUG_PRINT(DEBUG_DVI,(" %d %d %d %d %d %d '%.*s'",k,c,s,d,a,l,
 		 a+l,current+14));
 #endif
   }
@@ -95,7 +95,7 @@ void FontDef(unsigned char* command, void* parent)
       && tfontnump->fontp->s == s 
       && tfontnump->fontp->d == d 
       && strncmp(tfontnump->fontp->n,current+14,a+l) == 0) {
-    DEBUG_PRINT((DEBUG_DVI|DEBUG_VF,"\n  FONT %d:\tMatch found",k));
+    DEBUG_PRINT((DEBUG_DVI|DEBUG_VF),("\n  FONT %d:\tMatch found",k));
     return;
   }
   /* If not found, create new */
@@ -124,12 +124,12 @@ void FontDef(unsigned char* command, void* parent)
   }
   /* If found, set its number and return */
   if (tfontptr!=NULL) {
-    DEBUG_PRINT(((DEBUG_DVI|DEBUG_VF),"\n  FONT %d:\tMatch found, number set",k));
+    DEBUG_PRINT((DEBUG_DVI|DEBUG_VF),("\n  FONT %d:\tMatch found, number set",k));
     tfontnump->fontp = tfontptr; 
     return;
   }
 
-  DEBUG_PRINT(((DEBUG_DVI|DEBUG_VF),"\n  FONT %d:\tNew entry created",k));
+  DEBUG_PRINT((DEBUG_DVI|DEBUG_VF),("\n  FONT %d:\tNew entry created",k));
   /* No fitting font found, create new entry. */
   if ((tfontptr = calloc(1,sizeof(struct font_entry))) == NULL)
     Fatal("can't malloc space for font_entry");
@@ -180,7 +180,7 @@ void FontFind(struct font_entry * tfontptr)
   dpi = kpse_magstep_fix ((unsigned) (tfontptr->font_mag / 5.0 + .5),
 			resolution, NULL);
   tfontptr->font_mag = dpi * 5; /* save correct dpi */
-  DEBUG_PRINT((DEBUG_DVI,"\n  FIND FONT:\t%s %d",tfontptr->n,dpi));
+  DEBUG_PRINT(DEBUG_DVI,("\n  FIND FONT:\t%s %d",tfontptr->n,dpi));
 
   name = kpse_find_vf (tfontptr->n);
   if (name!=NULL) {
@@ -189,7 +189,7 @@ void FontFind(struct font_entry * tfontptr)
     InitVF(tfontptr);
   }
 #ifdef HAVE_FT2
-  if (freetype && name==NULL) {
+  if ((flags & USE_FREETYPE) && name==NULL) {
     psfile = FindPSFontMap(tfontptr->n, &encoding, &transform);
     if (psfile!=NULL) {
       name = kpse_find_ft(psfile);
@@ -226,14 +226,18 @@ void FontFind(struct font_entry * tfontptr)
       free (name);
       
       if (!FILESTRCASEEQ (tfontptr->n, font_ret.name)) {
+	flags |= PAGE_GAVE_WARN;
 	Warning("font %s not found, using %s at %d instead.\n",
 		tfontptr->n, font_ret.name, font_ret.dpi);
 	tfontptr->c = 0; /* no checksum warning */
-      } else if (!kpse_bitmap_tolerance ((double)font_ret.dpi, (double) dpi))
+      } else if (!kpse_bitmap_tolerance ((double)font_ret.dpi, (double) dpi)) {
+	flags |= PAGE_GAVE_WARN;
 	Warning("font %s at %d not found, using %d instead.\n",
 		tfontptr->name, dpi, font_ret.dpi);
+      }
       InitPK(tfontptr);
     } else {
+      flags |= PAGE_GAVE_WARN;
       Warning("font %s at %u not found, characters will be left blank.\n",
 	      tfontptr->n, dpi);
       tfontptr->filedes = 0;
@@ -269,11 +273,12 @@ void DoneFont(struct font_entry *tfontp)
   case FONT_TYPE_VF:
     DoneVF(tfontp);
     break;
+#ifdef HAVE_FT2
   case FONT_TYPE_FT:
     DoneFT(tfontp);
     break;
+#endif
   }
-  free(tfontp);
 }
 
 
@@ -297,8 +302,8 @@ void ClearFonts(void)
     free(hfontptr);
     hfontptr=tmp;
   }
-  FreeFontNumP(dvi->fontnump);
-
+  if (dvi!=NULL)
+    FreeFontNumP(dvi->fontnump);
 }
 
 /*-->SetFntNum*/
